;; CC41A - CC54A : Lenguajes de Programación
;; Óscar Álvarez - Richard Ibarra


;; d/dx : (number -> number) -> (number -> number)
;; retorna df/dx (x) 
(define (d/dx f)
  (define dx 0.0001)
  (lambda (x)
    (/ (- (f (+ x dx)) (f x)) dx)
    )
  )


;; diff-square : number -> number
;; d/dx (x²) = 2x
(define diff-square (d/dx 
                     (lambda (x) (* x x)) ;; x²
                     )
  )
(diff-square 2)                                        ;; Cercano a 4. Entre más pequeño dx, más cercano a 4.

;; diff-dsin : number -> number
;; d/dx sin(2x) = 2cos(2x)
(define diff-dsin (d/dx (lambda (x) (sin (* 2 x)))))


;; f-compose : (X->Y) (Y->Z) -> (X->Z)
;; (f-compose f g) -> fog
(define (f-compose f g)
  (lambda (x) (f (g x))
    )
  )

;; f-n : (X -> X) number -> (X -> X)
;; (f-n f n) -> f^n
(define (f-n f n)
  (cond
    ((= n 0) (lambda (x) x))
    ((= n 1) f)
    (else (f-compose f (f-n f (- n 1))))
    )
  )

;; second : list -> atom
(define second (f-compose car cdr))

;; third : list -> atom
(define third (f-compose car (f-compose cdr cdr)))

;; forth : list -> atom
(define fourth (f-compose car (f-compose cdr (f-compose cdr cdr))))

;; (second '(1 2 3 4)) -> 2
;; (third '(1 2 3 4))  -> 3
;; (fourth '(1 2 3 4)) -> 4

;; -listref : list number -> atom
;; similar a l[i] ...
;; a contar de l[0]
;; cdr : listOf(X) -> listOf(X)
;; car : listOf(X) -> X
(define (-listref l n)
  ((f-compose car (f-n cdr n)) l)
  )
  

(define -l '(1 2 3))
(-listref -l 0)
(-listref -l 1)
(-listref -l 2)   
